import mock
from kinto_http.exceptions import KintoException
from amo2kinto.kinto import get_kinto_records


def test_get_kinto_records_try_to_create_the_bucket():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value.status_code = 201
    get_kinto_records(kinto_client, mock.sentinel.bucket,
                      mock.sentinel.collection, mock.sentinel.permissions)

    kinto_client.create_bucket.assert_called_with(id=mock.sentinel.bucket,
                                                  if_not_exists=True)


def test_get_kinto_records_try_to_create_the_bucket_and_keep_going_on_403():
    kinto_client = mock.MagicMock()
    Http403 = mock.MagicMock()
    Http403.response.status_code = 403
    kinto_client.create_bucket.side_effect = KintoException(exception=Http403)
    get_kinto_records(kinto_client, mock.sentinel.bucket,
                      mock.sentinel.collection, mock.sentinel.permissions)

    kinto_client.create_bucket.assert_called_with(id=mock.sentinel.bucket,
                                                  if_not_exists=True)
    kinto_client.create_collection.assert_called_with(
        id=mock.sentinel.collection, bucket=mock.sentinel.bucket,
        permissions=mock.sentinel.permissions, if_not_exists=True)


def test_get_kinto_records_try_to_create_the_collection_with_permissions():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value.status_code = 201
    get_kinto_records(kinto_client, mock.sentinel.bucket,
                      mock.sentinel.collection, mock.sentinel.permissions)

    kinto_client.create_collection.assert_called_with(
        id=mock.sentinel.collection, bucket=mock.sentinel.bucket,
        permissions=mock.sentinel.permissions, if_not_exists=True)


def test_get_kinto_records_try_to_create_the_collection_and_keep_going_on_403():
    kinto_client = mock.MagicMock()
    Http403 = mock.MagicMock()
    Http403.response.status_code = 403
    kinto_client.create_collection.side_effect = KintoException(exception=Http403)
    get_kinto_records(kinto_client, mock.sentinel.bucket,
                      mock.sentinel.collection, mock.sentinel.permissions)

    kinto_client.get_collection.assert_called_with(id=mock.sentinel.collection,
                                                   bucket=mock.sentinel.bucket)


def test_get_kinto_records_gets_a_list_of_records():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value.status_code = 201
    get_kinto_records(kinto_client, mock.sentinel.bucket,
                      mock.sentinel.collection, mock.sentinel.permissions)

    kinto_client.get_records.assert_called_with(
        bucket=mock.sentinel.bucket, collection=mock.sentinel.collection)


def test_get_kinto_records_try_to_create_the_collection_with_schema():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value.status_code = 201
    kinto_client.create_collection.return_value.json.return_value = {
        "data": {
            "schema": {},
            "uiSchema": {},
            "displayFields": {}
        }
    }
    get_kinto_records(kinto_client,
                      mock.sentinel.bucket,
                      mock.sentinel.collection,
                      mock.sentinel.permissions,
                      config={"schema": {'foo': 'bar'}})

    kinto_client.patch_collection.assert_called_with(
        bucket=mock.sentinel.bucket,
        id=mock.sentinel.collection,
        data={"schema": {"foo": "bar"}})


def test_get_kinto_records_try_to_update_the_collection_schema():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value = {
        "details": {
            "existing": {
                "schema": {},
                "uiSchema": {},
                "displayFields": {}
            }
        }
    }
    get_kinto_records(kinto_client,
                      mock.sentinel.bucket,
                      mock.sentinel.collection,
                      mock.sentinel.permissions,
                      config={"schema": {'foo': 'bar'}})

    kinto_client.patch_collection.assert_called_with(
        bucket=mock.sentinel.bucket,
        id=mock.sentinel.collection,
        data={"schema": {"foo": "bar"}})


def test_get_kinto_records_doesnt_update_the_collection_schema_if_identical():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value = {
        "details": {
            "existing": {
                "schema": {"foo": "bar"},
                "uiSchema": {"ui": "bar"},
                "displayFields": ["display", "bar"],
            }
        }
    }
    get_kinto_records(kinto_client,
                      mock.sentinel.bucket,
                      mock.sentinel.collection,
                      mock.sentinel.permissions,
                      config={
                          "schema": {"foo": "bar"},
                          "uiSchema": {"ui": "bar"},
                          "displayFields": ["display", "bar"],
                      })

    assert not kinto_client.patch_collection.called


def test_get_kinto_records_does_update_if_it_has_created_it():
    kinto_client = mock.MagicMock()
    kinto_client.create_collection.return_value = {
        "data": {
            "schema": {}
        }
    }
    get_kinto_records(kinto_client,
                      mock.sentinel.bucket,
                      mock.sentinel.collection,
                      mock.sentinel.permissions,
                      config={"schema": {'foo': 'bar'}})

    assert kinto_client.patch_collection.called



# This file is taken as is from Olympia addons-server source code.
#
# https://github.com/mozilla/addons-server/
# - src/olympia/versions/compare.py
import re

MAXVERSION = 2 ** 63 - 1

version_re = re.compile(r"""(?P<major>\d+|\*)      # major (x in x.y)
                            \.?(?P<minor1>\d+|\*)? # minor1 (y in x.y)
                            \.?(?P<minor2>\d+|\*)? # minor2 (z in x.y.z)
                            \.?(?P<minor3>\d+|\*)? # minor3 (w in x.y.z.w)
                            (?P<alpha>[a|b]?)      # alpha/beta
                            (?P<alpha_ver>\d*)     # alpha/beta version
                            (?P<pre>pre)?          # pre release
                            (?P<pre_ver>\d)?       # pre release version
                        """,
                        re.VERBOSE)


def version_dict(version):
    """Turn a version string into a dict with major/minor/... info."""
    match = version_re.match(str(version) or '')
    letters = 'alpha pre'.split()
    numbers = 'major minor1 minor2 minor3 alpha_ver pre_ver'.split()
    if match:
        d = match.groupdict()
        for letter in letters:
            d[letter] = d[letter] if d[letter] else None
        for num in numbers:
            if d[num] == '*':
                d[num] = 99
            else:
                d[num] = int(d[num]) if d[num] else None
    else:
        d = dict((k, None) for k in numbers)
        d.update((k, None) for k in letters)
    return d


def version_int(version):
    if isinstance(version, bytes):  # pragma: no cover
        version = version.decode('utf-8')

    d = version_dict(version)
    for key in ['alpha_ver', 'major', 'minor1', 'minor2', 'minor3',
                'pre_ver']:
        if not d[key]:
            d[key] = 0
    atrans = {'a': 0, 'b': 1}
    d['alpha'] = atrans.get(d['alpha'], 2)
    d['pre'] = 0 if d['pre'] else 1

    v = "%d%02d%02d%02d%d%02d%d%02d" % (
        d['major'], d['minor1'], d['minor2'], d['minor3'], d['alpha'],
        d['alpha_ver'], d['pre'], d['pre_ver'])
    return min(int(v), MAXVERSION)



import os

# Destination directory
TARGET_DIR = os.path.abspath('results')

# Addons Server defaults
ADDONS_SERVER = 'https://addons-dev.allizom.org/'

# Kinto server defaults
KINTO_SERVER = 'http://localhost:8888/v1'
AUTH = ('mark', 'p4ssw0rd')
COLLECTION_PERMISSIONS = {}

# Buckets name default
ADDONS_BUCKET = u'staging'
ADDONS_COLLECTION = u'addons'
CERT_BUCKET = u'staging'
CERT_COLLECTION = u'certificates'
GFX_BUCKET = u'staging'
GFX_COLLECTION = u'gfx'
PLUGINS_BUCKET = u'staging'
PLUGINS_COLLECTION = u'plugins'

DESTINATION_BUCKET = 'blocklists'

# APP ID
FIREFOX_APPID = '{ec8030f7-c20a-464f-9b0e-13a3a9e97384}'
THUNDERBIRD_APPID = '{3550f703-e582-4d05-9a08-453d09bdfdc6}'
SEAMONKEY_APPID = '{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}'
ANDROID_APPID = '{aa3c5121-dab2-40e2-81ca-7ea25febc110}'



import mock
import unittest

from jinja2 import Template

from amo2kinto import constants
from amo2kinto.generator import (
    iso2human, get_template, generate, generate_index, generate_record, get_record_filename, main)


ADDONS_DATA = {
    "id": "e3e8f123-588d-0f73-63d8-93bdfc6ae8e2",
    "last_modified": 1368430987148,
    "guid": "sqlmoz@facebook.com",
    "blockID": "i454",
    "details": {
        "who": "All Firefox users who have this extension installed.",
        "created": "2013-05-13T09:43:07Z",
        "name": "Mozilla Service Pack (malware)",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=871610",
        "why": "This extension is malware posing as Mozilla software. "
        "It hijacks Facebook accounts and spams other Facebook users."
    },
    "versionRange": [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"},
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }, {
        "targetApplication": [
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }, {
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 0
    }],
    "prefs": ["test.blocklist"]
}


PLUGIN_DATA = {
    "id": "6a1b6dfe-f463-3061-e8f8-6e896ccf2a8a",
    "last_modified": 1301581706645,
    "blockID": "p26",
    "matchName": "^Yahoo Application State Plugin$",
    "matchFilename": "npYState.dll",
    "matchDescription": "^Yahoo Application State Plugin$",
    "infoURL": "https://get.adobe.com/flashplayer/",
    "details": {
        "who": "Users of all versions of Yahoo Application State Plugin "
        "for Firefox 3 and later.\r\n\r\nUsers of all versions of Yahoo "
        "Application State Plugin for SeaMonkey 1.0.0.5 and later.",
        "created": "2011-03-31T16:28:26Z",
        "name": "Yahoo Application State Plugin",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=421993",
        "why": "This plugin causes a high volume of Firefox and SeaMonkey "
        "crashes."
    },
    "versionRange": [{
        "minVersion": "0",
        "maxVersion": "4.1.10328.0",
        "severity": 0,
        "vulnerabilityStatus": 1,
        "targetApplication": [{
            "minVersion": "3.0a1",
            "guid": constants.FIREFOX_APPID,
            "maxVersion": "3.*"
        }]
    }]
}


def test_iso2human():
    assert iso2human(ADDONS_DATA['details']['created']) == 'May 13, 2013'
    assert iso2human(PLUGIN_DATA['details']['created']) == 'Mar 31, 2011'


def test_get_template():
    assert isinstance(get_template('collection.tpl'), Template)
    assert isinstance(get_template('record.tpl'), Template)


def test_generate_index():
    records = [ADDONS_DATA, PLUGIN_DATA]
    template = get_template('collection.tpl')
    f = mock.mock_open()
    with mock.patch('amo2kinto.generator.open', f, create=True):
        generate_index(records, template, 'tmp')
        f.assert_called_once_with('tmp/index.html', 'w')
        handle = f()
        assert handle.write.call_count == 1


def test_get_record_filename_uses_blockID():
    record_id = '9cff5990-47fe-4732-afaa-d2d82094baa1'
    assert get_record_filename({'blockID': 'i454', 'id': record_id}) == 'i454.html'


def test_get_record_filename_uses_record_id():
    record_id = '9cff5990-47fe-4732-afaa-d2d82094baa1'
    assert get_record_filename({'id': record_id}) == '%s.html' % record_id


def test_generate_record():
    template = get_template('record.tpl')
    f = mock.mock_open()
    with mock.patch('amo2kinto.generator.os.makedirs'):
        with mock.patch('amo2kinto.generator.open', f, create=True):
            generate_record(ADDONS_DATA, template, 'tmp', get_record_filename)
            f.assert_called_once_with('tmp/i454.html', 'w')
            handle = f()
            assert handle.write.call_count == 1


def test_generate_grabs_addons_and_plugins_records():
    kinto_client = mock.MagicMock()
    kinto_client.get_records.side_effect = [[ADDONS_DATA], [PLUGIN_DATA]]
    collections = ['/buckets/blocklists/collections/addons',
                   '/buckets/blocklists/collections/plugins']

    collection_template = get_template('collection.tpl')
    record_template = get_template('record.tpl')

    with mock.patch('amo2kinto.generator.os.makedirs') as mocked_makedirs:
        with mock.patch('amo2kinto.generator.get_template',
                        side_effect=[collection_template, record_template]):
            with mock.patch('amo2kinto.generator.generate_index') as gi:
                with mock.patch('amo2kinto.generator.generate_record') as gr:
                    generate(kinto_client, collections, 'tmp', 'collection.tpl', 'record.tpl')

                    assert gi.call_count == 1
                    gi.assert_called_with([ADDONS_DATA, PLUGIN_DATA], collection_template, 'tmp')

                    assert gr.call_count == 2
                    gr.assert_any_call(ADDONS_DATA, record_template, 'tmp', get_record_filename)
                    gr.assert_any_call(PLUGIN_DATA, record_template, 'tmp', get_record_filename)

                    mocked_makedirs.assert_called_with('tmp')


def test_generate_uses_last_modified_if_created_is_missing():
    kinto_client = mock.MagicMock()
    data = ADDONS_DATA.copy()
    del data['details']['created']
    kinto_client.get_records.return_value = [data]
    collections = ['/buckets/blocklists/collections/addons']

    with mock.patch('amo2kinto.generator.os.makedirs'):
        f = mock.mock_open()
        with mock.patch('amo2kinto.generator.open', f, create=True):
            generate(kinto_client, collections, 'tmp', 'collection.tpl', 'record.tpl')

            assert f.return_value.write.call_count == 2

            # Present in index
            assert b'May 13, 2013' in f.return_value.write.call_args_list[0][0][0]

            # Present in the record file
            assert b'May 13, 2013' in f.return_value.write.call_args_list[1][0][0]


class TestMain(unittest.TestCase):
    def setUp(self):
        p = mock.patch('kinto_http.cli_utils.Client')
        self.MockedClient = p.start()
        self.MockedClient.return_value.get_records.return_value = []
        self.addCleanup(p.stop)

        p = mock.patch('amo2kinto.generator.os.makedirs')
        self.mocked_makedirs = p.start()
        self.addCleanup(p.stop)

        p = mock.patch('amo2kinto.generator.open', create=True)
        self.mocked_open = p.start()
        self.addCleanup(p.stop)

    def assert_arguments(self, kinto_client, **kwargs):
        kwargs.setdefault('kinto_server', constants.KINTO_SERVER)
        kwargs.setdefault('auth', constants.AUTH)
        kwargs.setdefault('bucket', constants.DESTINATION_BUCKET)
        kwargs.setdefault('addons_collection', constants.ADDONS_COLLECTION)
        kwargs.setdefault('plugins_collection', constants.PLUGINS_COLLECTION)
        kwargs.setdefault('target_dir', constants.TARGET_DIR)

        kinto_client.assert_called_with(server_url=kwargs['kinto_server'],
                                        auth=kwargs['auth'],
                                        bucket=kwargs['bucket'],
                                        collection=None,
                                        retry=0, retry_after=None)

        addons_arguments = {
            'bucket': kwargs['bucket'],
            'collection': kwargs['addons_collection'],
        }
        kinto_client.return_value.get_records.assert_any_call(
            **addons_arguments)

        plugins_arguments = {
            'bucket': kwargs['bucket'],
            'collection': kwargs['plugins_collection'],
        }

        kinto_client.return_value.get_records.assert_any_call(
            **plugins_arguments)

    def test_main_default(self):
        # let's check that main() parsing uses our defaults
        main([])
        self.assert_arguments(self.MockedClient)

    def test_main_custom_server(self):
        main(['-s', 'http://yeah'])
        self.assert_arguments(self.MockedClient, kinto_server='http://yeah')

    def test_can_define_the_addons_collection(self):
        main(['--bucket', 'bucket',
              '--addons-collection', 'collection'])
        self.assert_arguments(self.MockedClient,
                              bucket='bucket',
                              addons_collection='collection')

    def test_can_define_the_plugins_collection(self):
        main(['--bucket', 'bucket',
              '--plugins-collection', 'collection'])
        self.assert_arguments(self.MockedClient,
                              bucket='bucket',
                              plugins_collection='collection')

    def test_can_define_the_auth_credentials(self):
        main(['--auth', 'user:pass'])
        self.assert_arguments(self.MockedClient, auth=('user', 'pass'))

    def test_can_define_the_output_directory(self):
        main(['--target-dir', 'file'])
        self.assert_arguments(self.MockedClient)

        self.mocked_open.assert_called_with('file/index.html', 'w')



import mock
import unittest

from contextlib import contextmanager
from amo2kinto.synchronize import get_diff, push_changes


def test_get_diff():
    source = [{'id': 1, 'val': 2}, {'id': 2, 'val': 3}, {'id': 4, 'val': 4}]
    dest = [{'id': 2, 'val': 3}, {'id': 3, 'val': 4}, {'id': 4, 'val': 5}]
    assert get_diff(source, dest) == (
        [{'id': 1, 'val': 2}],
        [{'id': 4, 'val': 4}],
        [{'id': 3, 'val': 4}])


class SynchronizeTest(unittest.TestCase):

    def setUp(self):
        self.mocked_batch = mock.MagicMock()

        @contextmanager
        def mocked_context_manager(bucket, collection):
            yield self.mocked_batch

        self.kinto_client = mock.MagicMock()
        self.kinto_client.batch = mocked_context_manager
        self.bucket = mock.sentinel.bucket
        self.collection = mock.sentinel.collection

    def test_synchronize_uses_kinto_client_for_signature(self):
        push_changes(([{'id': 1, 'val': 2}],
                      [{'id': 4, 'val': 4}],
                      [{'id': 3, 'val': 4}]),
                     self.kinto_client, self.bucket, self.collection)

        self.kinto_client.patch_collection.assert_any_call(
            bucket=mock.sentinel.bucket,
            id=mock.sentinel.collection,
            data={'status': 'to-review'})

        self.kinto_client.patch_collection.assert_any_call(
            bucket=mock.sentinel.bucket,
            id=mock.sentinel.collection,
            data={'status': 'to-sign'})

    def test_synchronize_uses_editor_and_reviewer_clients(self):
        editor_client = mock.MagicMock()
        reviewer_client = mock.MagicMock()

        push_changes(([{'id': 1, 'val': 2}],
                      [{'id': 4, 'val': 4}],
                      [{'id': 3, 'val': 4}]),
                     self.kinto_client, self.bucket, self.collection,
                     editor_client, reviewer_client)

        editor_client.patch_collection.assert_called_with(
            bucket=mock.sentinel.bucket,
            id=mock.sentinel.collection,
            data={'status': 'to-review'})

        reviewer_client.patch_collection.assert_called_with(
            bucket=mock.sentinel.bucket,
            id=mock.sentinel.collection,
            data={'status': 'to-sign'})

    def test_synchronize_create_the_batch_request(self):
        push_changes(([{'id': 1, 'val': 2}],
                      [{'id': 4, 'val': 4}],
                      [{'id': 3, 'val': 4}]),
                     self.kinto_client, self.bucket, self.collection)

        self.mocked_batch.create_record.assert_called_with(
            data={'id': 1, 'val': 2, 'enabled': True})
        self.mocked_batch.update_record.assert_called_with(
            data={'id': 4, 'enabled': True, 'val': 4})
        self.mocked_batch.delete_record.assert_called_with(id=3)

    def test_synchronize_triggers_the_signature(self):
        push_changes(([{'id': 1, 'val': 2}],
                      [{'id': 4, 'val': 4}],
                      [{'id': 3, 'val': 4}]),
                     self.kinto_client, self.bucket, self.collection)

        self.kinto_client.patch_collection.assert_called_with(
            data={'status': 'to-sign'},
            bucket=mock.sentinel.bucket, id=mock.sentinel.collection)

    def test_synchronize_does_not_triggers_the_signer_on_empty_changes(self):
        push_changes(([], [], []),
                     self.kinto_client,
                     self.bucket,
                     self.collection)

        self.assertFalse(self.kinto_client.patch_collection.called)



from functools import partial
from kinto_http.exceptions import KintoException


def update_schema_if_mandatory(response, config, patch_collection):
    if 'details' in response:
        collection_schema = {
            'schema': response['details']['existing'].get('schema'),
            'uiSchema': response['details']['existing'].get('uiSchema'),
            'displayFields': response['details']['existing'].get(
                'displayFields'),
        }
    else:
        collection_schema = {
            'schema': response['data'].get('schema'),
            'uiSchema': response['data'].get('uiSchema'),
            'displayFields': response['data'].get('displayFields'),
        }

    if config and collection_schema != config:
        patch_collection(data=config)


def get_kinto_records(kinto_client, bucket, collection, permissions,
                      config=None):
    """Return all the kinto records for this bucket/collection."""
    # Create bucket if needed
    try:
        kinto_client.create_bucket(id=bucket, if_not_exists=True)
    except KintoException as e:
        if hasattr(e, 'response') and e.response.status_code == 403:
            # The user cannot create buckets on this server, ignore the creation.
            pass

    try:
        response = kinto_client.create_collection(
            id=collection, bucket=bucket, permissions=permissions, if_not_exists=True)
    except KintoException as e:
        if hasattr(e, 'response') and e.response.status_code == 403:
            # The user cannot create collection on this bucket, ignore the creation.
            pass
        response = kinto_client.get_collection(id=collection, bucket=bucket)

    patch_collection = partial(kinto_client.patch_collection,
                               bucket=bucket, id=collection)

    update_schema_if_mandatory(response, config, patch_collection)

    return kinto_client.get_records(bucket=bucket, collection=collection)



import sys
from collections import OrderedDict
from lxml import etree
from kinto_http import cli_utils
from . import constants
from .compare import version_int

from .logger import logger


def build_version_range(root, item, app_id, app_ver=None):
    for version in item['versionRange']:
        is_version_related_to_app = (
            not version.get('targetApplication') or
            any(tA for tA in version.get('targetApplication', [])
                if not tA.get('guid') or tA.get('guid') == app_id))

        if is_version_related_to_app:
            versionRange = etree.SubElement(
                root, 'versionRange')

            for field in ['minVersion', 'maxVersion', 'severity']:
                value = version.get(field)
                if value:
                    versionRange.set(field, str(value))

            has_targetApplication = (
                'targetApplication' in version and version['targetApplication']
            )

            if has_targetApplication:
                for tA in version['targetApplication']:
                    is_targetApp_related = (
                        not tA['guid'] or tA['guid'] == app_id
                    )
                    if is_targetApp_related:
                        targetApplication = etree.SubElement(
                            versionRange, 'targetApplication',
                            id=tA['guid'])
                        etree.SubElement(
                            targetApplication, 'versionRange',
                            minVersion=tA['minVersion'],
                            maxVersion=tA['maxVersion'])


def is_related_to(item, app_id, app_ver=None):
    """Return True if the item relates to the given app_id (and app_ver, if passed)."""
    versionRange = item.get('versionRange')
    if not versionRange:
        return True

    for vR in versionRange:
        if not vR.get('targetApplication'):
            return True
        if get_related_targetApplication(vR, app_id, app_ver) is not None:
            return True
    return False


def get_related_targetApplication(vR, app_id, app_ver):
    """Return the first matching target application in this version range.
    Returns None if there are no target applications or no matching ones."""
    targetApplication = vR.get('targetApplication')
    if not targetApplication:
        return None

    for tA in targetApplication:
        guid = tA.get('guid')
        if not guid or guid == app_id:
            if not app_ver:
                return tA
            # We purposefully use maxVersion only, so that the blocklist contains items
            # whose minimum version is ahead of the version we get passed. This means
            # the blocklist we serve is "future-proof" for app upgrades.
            if between(version_int(app_ver), '0', tA.get('maxVersion', '*')):
                return tA

    return None


def should_include_certs(app_id=None, app_ver=None):
    """Whether we should include certificate blocklist items for this application version."""
    # As of Firefox 58, we definitely won't need the cert items in here.
    # They get fetched via a different means than the XML-based blocklist
    return app_id != constants.FIREFOX_APPID or version_int(app_ver) < version_int("58.0")


def write_addons_items(xml_tree, records, app_id, api_ver=3, app_ver=None):
    """Generate the addons blocklists.

    <emItem blockID="i372" id="5nc3QHFgcb@r06Ws9gvNNVRfH.com">
      <versionRange minVersion="0" maxVersion="*" severity="3">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange minVersion="39.0a1" maxVersion="*"/>
        </targetApplication>
      </versionRange>
      <prefs>
        <pref>browser.startup.homepage</pref>
        <pref>browser.search.defaultenginename</pref>
      </prefs>
    </emItem>
    """
    if not records:
        return

    emItems = etree.SubElement(xml_tree, 'emItems')
    groupby = {}
    for item in records:
        if is_related_to(item, app_id, app_ver):
            if item['guid'] in groupby:
                emItem = groupby[item['guid']]
                # When creating new records from the Kinto Admin we don't have proper blockID.
                if 'blockID' in item:
                    # Remove the first caracter which is the letter i to
                    # compare the numeric value i45 < i356.
                    current_blockID = item['blockID'][1:]
                    previous_blockID = emItem.attrib['blockID'][1:]
                    # Group by and keep the biggest blockID in the XML file.
                    if current_blockID > previous_blockID:
                        emItem.attrib['blockID'] = item['blockID']
            else:
                emItem = etree.SubElement(emItems, 'emItem',
                                          blockID=item.get('blockID', item['id']))
                groupby[item['guid']] = emItem
                prefs = etree.SubElement(emItem, 'prefs')
                for p in item['prefs']:
                    pref = etree.SubElement(prefs, 'pref')
                    pref.text = p

            # Set the add-on ID
            emItem.set('id', item['guid'])

            for field in ['name', 'os']:
                if field in item:
                    emItem.set(field, item[field])

            build_version_range(emItem, item, app_id)


def between(ver, min, max):
    if not (min and max):
        return True
    return version_int(min) < ver < version_int(max)


def write_plugin_items(xml_tree, records, app_id, api_ver=3, app_ver=None):
    """Generate the plugin blocklists.

    <pluginItem blockID="p422">
        <match name="filename" exp="JavaAppletPlugin\.plugin"/>
        <versionRange minVersion="Java 7 Update 16"
                      maxVersion="Java 7 Update 24"
                      severity="0" vulnerabilitystatus="1">
            <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
                <versionRange minVersion="17.0" maxVersion="*"/>
            </targetApplication>
        </versionRange>
    </pluginItem>
    """

    if not records:
        return

    pluginItems = etree.SubElement(xml_tree, 'pluginItems')
    for item in records:
        for versionRange in item.get('versionRange', []):
            if not versionRange.get('targetApplication'):
                add_plugin_item(pluginItems, item, versionRange,
                                app_id=app_id, api_ver=api_ver,
                                app_ver=app_ver)
            else:
                targetApplication = get_related_targetApplication(versionRange, app_id, app_ver)
                if targetApplication is not None:
                    add_plugin_item(pluginItems, item, versionRange, targetApplication,
                                    app_id=app_id, api_ver=api_ver,
                                    app_ver=app_ver)


def add_plugin_item(pluginItems, item, version, tA=None, app_id=None,
                    api_ver=3, app_ver=None):

    entry = etree.SubElement(pluginItems, 'pluginItem',
                             blockID=item.get('blockID', item['id']))

    for field in ['name', 'os', 'xpcomabi']:
        if field in item:
            entry.set(field, item[field])

    for xml_field in ['name', 'filename', 'description']:
        json_field = 'match%s' % xml_field.capitalize()
        if json_field in item:
            etree.SubElement(entry, 'match',
                             name=xml_field,
                             exp=item[json_field])

    if 'infoURL' in item:
        infoURL = etree.SubElement(entry, 'infoURL')
        infoURL.text = item['infoURL']

    minVersion = version.get('minVersion')
    maxVersion = version.get('maxVersion')
    severity = version.get('severity')
    vulnerabilityStatus = version.get('vulnerabilityStatus')

    app_guid = tA and tA.get('guid') or None

    kwargs = OrderedDict()
    force_empty_versionRange = False

    # Condition taken exactly as they were in addons-server code.
    if (severity or app_guid or (minVersion and maxVersion) or
            vulnerabilityStatus):
        if app_guid:
            if minVersion and maxVersion:
                kwargs['maxVersion'] = maxVersion
                kwargs['minVersion'] = minVersion
                if severity is not None:
                    kwargs['severity'] = str(severity)
                if vulnerabilityStatus:
                    kwargs['vulnerabilitystatus'] = str(vulnerabilityStatus)
            else:
                force_empty_versionRange = True
                if severity is not None:
                    kwargs['severity'] = str(severity)
                if vulnerabilityStatus is not None:
                    kwargs['vulnerabilitystatus'] = str(vulnerabilityStatus)

        elif api_ver > 2 and minVersion and maxVersion:
            kwargs['maxVersion'] = maxVersion
            kwargs['minVersion'] = minVersion
            if severity is not None:
                kwargs['severity'] = str(severity)
            if vulnerabilityStatus is not None:
                kwargs['vulnerabilitystatus'] = str(vulnerabilityStatus)

        elif severity and not (minVersion or maxVersion):
            kwargs['severity'] = str(severity)

        elif vulnerabilityStatus:
            if severity is not None:
                kwargs['severity'] = str(severity)
            kwargs['vulnerabilitystatus'] = str(vulnerabilityStatus)

    elif severity == 0:
        kwargs['severity'] = str(severity)

    versionRange_not_null = (
        len(kwargs.keys()) or (api_ver > 2 and tA and tA.get(
            'minVersion') and tA.get('maxVersion'))
    )

    if versionRange_not_null or force_empty_versionRange:
        versionRange = etree.SubElement(entry, 'versionRange', **kwargs)

    is_targetApplication_applicable = (api_ver > 2 and
                                       tA and
                                       tA.get('minVersion') and
                                       tA.get('maxVersion'))

    if is_targetApplication_applicable:
        targetApplication = etree.SubElement(
            versionRange, 'targetApplication',
            id=tA['guid'])
        etree.SubElement(targetApplication,
                         'versionRange',
                         minVersion=tA['minVersion'],
                         maxVersion=tA['maxVersion'])


def write_gfx_items(xml_tree, records, app_id, api_ver=3):
    """Generate the gfxBlacklistEntry.

    <gfxBlacklistEntry blockID="g35">
        <os>WINNT 6.1</os>
        <vendor>0x10de</vendor>
        <devices>
            <device>0x0a6c</device>
        </devices>
        <feature>DIRECT2D</feature>
        <featureStatus>BLOCKED_DRIVER_VERSION</featureStatus>
        <driverVersion>8.17.12.5896</driverVersion>
        <driverVersionComparator>LESS_THAN_OR_EQUAL</driverVersionComparator>
        <versionRange minVersion="3.2" maxVersion="3.4" />
    </gfxBlacklistEntry>
    """
    if not records:
        return

    gfxItems = etree.SubElement(xml_tree, 'gfxItems')
    for item in records:
        is_record_related = ('guid' not in item or item['guid'] == app_id)

        if is_record_related:
            entry = etree.SubElement(gfxItems, 'gfxBlacklistEntry',
                                     blockID=item.get('blockID', item['id']))
            fields = ['os', 'vendor', 'feature', 'featureStatus',
                      'driverVersion', 'driverVersionComparator']
            for field in fields:
                if field in item:
                    node = etree.SubElement(entry, field)
                    node.text = item[field]

            # Devices
            if item['devices']:
                devices = etree.SubElement(entry, 'devices')
                for d in item['devices']:
                    device = etree.SubElement(devices, 'device')
                    device.text = d

            if 'versionRange' in item:
                version = item['versionRange']
                versionRange = etree.SubElement(entry, 'versionRange')

                for field in ['minVersion', 'maxVersion']:
                    value = version.get(field)
                    if value:
                        versionRange.set(field, str(value))


def write_cert_items(xml_tree, records, api_ver=3, app_id=None, app_ver=None):
    """Generate the certificate blocklists.

    <certItem issuerName="MIGQMQswCQYD...IENB">
      <serialNumber>UoRGnb96CUDTxIqVry6LBg==</serialNumber>
    </certItem>

    or

    <certItem subject='MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5'
              pubKeyHash='VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8='>
    </certItem>
    """
    if not records or not should_include_certs(app_id, app_ver):
        return

    certItems = etree.SubElement(xml_tree, 'certItems')
    for item in records:
        if item.get('subject') and item.get('pubKeyHash'):
            cert = etree.SubElement(certItems, 'certItem',
                                    subject=item['subject'],
                                    pubKeyHash=item['pubKeyHash'])
        else:
            cert = etree.SubElement(certItems, 'certItem',
                                    issuerName=item['issuerName'])
            serialNumber = etree.SubElement(cert, 'serialNumber')
            serialNumber.text = item['serialNumber']


def main(args=None):
    parser = cli_utils.add_parser_options(
        description='Build a blocklists.xml file from Kinto blocklists.',
        default_collection=None,
        default_bucket=None,
        default_server=constants.KINTO_SERVER,
        default_auth=constants.AUTH,
        include_bucket=False,
        include_collection=False)

    parser.add_argument('--certificates-bucket',
                        help='Bucket name for certificates',
                        type=str, default=constants.CERT_BUCKET)

    parser.add_argument('--certificates-collection',
                        help='Collection name for certificates',
                        type=str, default=constants.CERT_COLLECTION)

    parser.add_argument('--gfx-bucket', help='Bucket name for gfx',
                        type=str, default=constants.GFX_BUCKET)

    parser.add_argument('--gfx-collection',
                        help='Collection name for gfx',
                        type=str, default=constants.GFX_COLLECTION)

    parser.add_argument('--addons-bucket', help='Bucket name for addons',
                        type=str, default=constants.ADDONS_BUCKET)

    parser.add_argument('--addons-collection',
                        help='Collection name for addon',
                        type=str, default=constants.ADDONS_COLLECTION)

    parser.add_argument('--plugins-bucket', help='Bucket name for plugins',
                        type=str, default=constants.PLUGINS_BUCKET)

    parser.add_argument('--plugins-collection',
                        help='Collection name for plugin',
                        type=str, default=constants.PLUGINS_COLLECTION)

    parser.add_argument('--api-version', help='Targeted blocklists.xml APP id',
                        type=int, default=3)

    parser.add_argument('--app', help='Targeted blocklists.xml APP id',
                        type=str, default=constants.FIREFOX_APPID)

    parser.add_argument('--app-version', help='The targetted app version',
                        type=str, default=None)

    # Choose where to write the file down.
    parser.add_argument('-o', '--out', help='Output XML file.',
                        type=str, default=None)

    args = parser.parse_args(args=args)

    cli_utils.setup_logger(logger, args)

    if not args.out:
        out_fd = sys.stdout
        close_out_fd = False
    else:
        out_fd = open(args.out, 'w+')
        close_out_fd = True

    client = cli_utils.create_client_from_args(args)

    last_update = 0
    # Retrieve the collection of records.
    try:
        addons_records = client.get_records(
            bucket=args.addons_bucket,
            collection=args.addons_collection,
            enabled=True,
            _sort="last_modified")
    except Exception:
        logger.warn(
            'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                bucket=args.addons_bucket,
                collection=args.addons_collection,
            )
        )
        addons_records = []

    if addons_records:
        last_update = addons_records[-1]['last_modified']

    try:
        plugin_records = client.get_records(
            bucket=args.plugins_bucket,
            collection=args.plugins_collection,
            enabled=True,
            _sort="last_modified")
    except Exception:
        logger.warn(
            'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                bucket=args.plugins_bucket,
                collection=args.plugins_collection,
            )
        )
        plugin_records = []

    if plugin_records:
        last_update = max(last_update, plugin_records[-1]['last_modified'])

    try:
        gfx_records = client.get_records(
            bucket=args.gfx_bucket,
            collection=args.gfx_collection,
            enabled=True,
            _sort="last_modified")
    except Exception:
        logger.warn(
            'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                bucket=args.gfx_bucket,
                collection=args.gfx_collection,
            )
        )
        gfx_records = []

    if gfx_records:
        last_update = max(last_update, gfx_records[-1]['last_modified'])

    cert_records = None
    if should_include_certs(args.app, args.app_version):
        try:
            cert_records = client.get_records(
                bucket=args.certificates_bucket,
                collection=args.certificates_collection,
                enabled=True,
                _sort="last_modified")
        except Exception:
            logger.warn(
                'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                    bucket=args.certificates_bucket,
                    collection=args.certificates_collection,
                )
            )
            cert_records = []

        if cert_records:
            last_update = max(last_update, cert_records[-1]['last_modified'])

    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='%s' % last_update
    )

    write_addons_items(xml_tree, addons_records,
                       api_ver=args.api_version,
                       app_id=args.app,
                       app_ver=args.app_version)
    write_plugin_items(xml_tree, plugin_records,
                       api_ver=args.api_version,
                       app_id=args.app,
                       app_ver=args.app_version)
    write_gfx_items(xml_tree, gfx_records,
                    api_ver=args.api_version,
                    app_id=args.app)
    write_cert_items(xml_tree, cert_records,
                     api_ver=args.api_version,
                     app_id=args.app,
                     app_ver=args.app_version)

    doc = etree.ElementTree(xml_tree)
    out_fd.write(etree.tostring(
        doc,
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8'))

    if close_out_fd:
        out_fd.close()



import json

from .logger import logger


def strip_keys(record, keys=['id', 'last_modified', 'enabled', 'schema']):
    return {key: value for key, value in record.items() if key not in keys}


def canonical_json(record):
    return json.dumps(strip_keys(record),
                      sort_keys=True,
                      separators=(',', ':'))


def get_diff(source, dest):
    """Get the diff between two records list in this order:
        - to_create
        - to_delete
    """
    # First build a dict from the lists, with the ID as the key.
    source_dict = {record['id']: record for record in source}
    dest_dict = {record['id']: record for record in dest}

    source_keys = set(source_dict.keys())
    dest_keys = set(dest_dict.keys())
    to_create = source_keys - dest_keys
    to_delete = dest_keys - source_keys
    to_update = set()

    to_check = source_keys - to_create - to_delete

    for record_id in to_check:
        # Make sure to remove properties that are part of kinto
        # records and not amo records.
        # Here we will compare the record properties ignoring:
        # ID, last_modified and enabled.
        new = canonical_json(source_dict[record_id])
        old = canonical_json(dest_dict[record_id])
        if new != old:
            to_update.add(record_id)

    return ([source_dict[k] for k in to_create],
            [source_dict[k] for k in to_update],
            [dest_dict[k] for k in to_delete])


def push_changes(diff, author_client, bucket, collection,
                 editor_client=None, reviewer_client=None):
    to_create, to_update, to_delete = diff

    if editor_client is None:
        editor_client = author_client

    if reviewer_client is None:
        reviewer_client = author_client

    logger.warn('Syncing to {}{}'.format(
        author_client.session.server_url,
        author_client.endpoints.get(
            'records', bucket=bucket, collection=collection)))

    logger.info('- {} records to create.'.format(len(to_create)))
    logger.info('- {} records to delete.'.format(len(to_delete)))

    with author_client.batch(bucket=bucket, collection=collection) as batch:
        for record in to_delete:
            batch.delete_record(id=record['id'])
        for record in to_create:
            # Records are enabled by default.
            record['enabled'] = True
            batch.create_record(data=record)
        for record in to_update:
            # Make sure the record is correcly activated.
            record['enabled'] = True
            batch.update_record(data=record)

    if to_create or to_update or to_delete:
        logger.info('Request review.')
        editor_client.patch_collection(data={'status': 'to-review'},
                                       bucket=bucket, id=collection)
        logger.info('Approve and trigger the signature.')
        reviewer_client.patch_collection(data={'status': 'to-sign'},
                                         bucket=bucket, id=collection)

    logger.info('Done!')



from amo2kinto.compare import MAXVERSION, version_int, version_dict


def test_version_int():
    """Tests that version_int. Corrects our versions."""
    assert version_int('3.5.0a1pre2') == 3050000001002
    assert version_int('') == 200100
    assert version_int('0') == 200100
    assert version_int('*') == 99000000200100
    assert version_int(MAXVERSION) == MAXVERSION
    assert version_int(MAXVERSION + 1) == MAXVERSION
    assert version_int('9999999') == MAXVERSION


def test_version_int_compare():
    assert version_int('3.6.*') == version_int('3.6.99')
    assert version_int('3.6.*') > version_int('3.6.8')


def test_version_asterix_compare():
    assert version_int('*') == version_int('99')
    assert version_int('98.*') < version_int('*')
    assert version_int('5.*') == version_int('5.99')
    assert version_int('5.*') > version_int('5.0.*')


def test_version_dict():
    assert version_dict('5.0') == (
        {'major': 5,
         'minor1': 0,
         'minor2': None,
         'minor3': None,
         'alpha': None,
         'alpha_ver': None,
         'pre': None,
         'pre_ver': None})


def test_version_int_unicode():
    assert version_int(u'\u2322 ugh stephend') == 200100



import mock
import unittest
from io import StringIO
from lxml import etree

from amo2kinto import constants, exporter, compare

ADDONS_DATA = {
    "id": "e3e8f123-588d-0f73-63d8-93bdfc6ae8e2",
    "last_modified": "1368430987148",
    "guid": "sqlmoz@facebook.com",
    "blockID": "i454",
    "details": {
        "who": "All Firefox users who have this extension installed.",
        "created": "2013-05-13T09:43:07Z",
        "name": "Mozilla Service Pack (malware)",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=871610",
        "why": "This extension is malware posing as Mozilla software. "
        "It hijacks Facebook accounts and spams other Facebook users."
    },
    "versionRange": [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"},
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }, {
        "targetApplication": [
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }, {
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 0
    }],
    "prefs": ["test.blocklist"]
}


def test_addon_record():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_addons_items(xml_tree, [ADDONS_DATA],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems>
    <emItem blockID="i454" id="sqlmoz@facebook.com">
      <prefs>
        <pref>test.blocklist</pref>
      </prefs>
      <versionRange minVersion="0" maxVersion="*" severity="3">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
        </targetApplication>
      </versionRange>
      <versionRange minVersion="0" maxVersion="*"/>
    </emItem>
  </emItems>
</blocklist>
""".decode('utf-8')


def test_addon_record_with_no_blockID():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = ADDONS_DATA.copy()
    del data['blockID']
    exporter.write_addons_items(xml_tree, [data],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems>
    <emItem blockID="e3e8f123-588d-0f73-63d8-93bdfc6ae8e2" id="sqlmoz@facebook.com">
      <prefs>
        <pref>test.blocklist</pref>
      </prefs>
      <versionRange minVersion="0" maxVersion="*" severity="3">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
        </targetApplication>
      </versionRange>
      <versionRange minVersion="0" maxVersion="*"/>
    </emItem>
  </emItems>
</blocklist>
""".decode('utf-8')


def test_addon_record_with_no_version_range_info():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = ADDONS_DATA.copy()
    data['versionRange'] = []

    exporter.write_addons_items(xml_tree, [data],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems>
    <emItem blockID="i454" id="sqlmoz@facebook.com">
      <prefs>
        <pref>test.blocklist</pref>
      </prefs>
    </emItem>
  </emItems>
</blocklist>
""".decode('utf-8')


def test_addon_record_with_no_targetApplication_info():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = ADDONS_DATA.copy()
    data['name'] = "Mozilla Service Pack (malware)"
    data['os'] = "WINNT"
    data['versionRange'] = [{
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 0
    }]

    exporter.write_addons_items(xml_tree, [data],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems>
    <emItem blockID="i454" id="sqlmoz@facebook.com" \
name="Mozilla Service Pack (malware)" os="WINNT">
      <prefs>
        <pref>test.blocklist</pref>
      </prefs>
      <versionRange minVersion="0" maxVersion="*"/>
    </emItem>
  </emItems>
</blocklist>
""".decode('utf-8')


def test_addon_record_with_no_targetApplication_matching():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = ADDONS_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }]

    exporter.write_addons_items(xml_tree, [data],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems/>
</blocklist>
""".decode('utf-8')


def test_addon_record_with_no_targetApplication_version_matching():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = ADDONS_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }]

    exporter.write_addons_items(xml_tree, [data],
                                constants.FIREFOX_APPID, app_ver="3.7")

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems/>
</blocklist>
""".decode('utf-8')


def test_addon_record_with_future_targetApplication_version_matching():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_addons_items(xml_tree, [ADDONS_DATA],
                                constants.FIREFOX_APPID, app_ver="3.1")

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems>
    <emItem blockID="i454" id="sqlmoz@facebook.com">
      <prefs>
        <pref>test.blocklist</pref>
      </prefs>
      <versionRange minVersion="0" maxVersion="*" severity="3">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
        </targetApplication>
      </versionRange>
      <versionRange minVersion="0" maxVersion="*"/>
    </emItem>
  </emItems>
</blocklist>
""".decode('utf-8')


def test_addon_record_group_by_blockID():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = ADDONS_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }]

    data2 = ADDONS_DATA.copy()
    data2['blockID'] = 'i586'
    data2['versionRange'] = [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 2
    }]

    exporter.write_addons_items(xml_tree, [data, data2],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <emItems>
    <emItem blockID="i586" id="sqlmoz@facebook.com">
      <prefs>
        <pref>test.blocklist</pref>
      </prefs>
      <versionRange minVersion="0" maxVersion="*" severity="3">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange maxVersion="1.4" minVersion="1.2"/>
        </targetApplication>
      </versionRange>
      <versionRange minVersion="0" maxVersion="*" severity="2">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
        </targetApplication>
      </versionRange>
    </emItem>
  </emItems>
</blocklist>
""".decode('utf-8')


PLUGIN_DATA = {
    "id": "6a1b6dfe-f463-3061-e8f8-6e896ccf2a8a",
    "last_modified": "1301581706645",
    "blockID": "p26",
    "matchName": "^Yahoo Application State Plugin$",
    "matchFilename": "npYState.dll",
    "matchDescription": "^Yahoo Application State Plugin$",
    "infoURL": "https://get.adobe.com/flashplayer/",
    "details": {
        "who": "Users of all versions of Yahoo Application State Plugin "
        "for Firefox 3 and later.\r\n\r\nUsers of all versions of Yahoo "
        "Application State Plugin for SeaMonkey 1.0.0.5 and later.",
        "created": "2011-03-31T16:28:26Z",
        "name": "Yahoo Application State Plugin",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=421993",
        "why": "This plugin causes a high volume of Firefox and SeaMonkey "
        "crashes."
    },
    "versionRange": [{
        "minVersion": "0",
        "maxVersion": "4.1.10328.0",
        "severity": 0,
        "vulnerabilityStatus": 1,
        "targetApplication": [{
            "minVersion": "3.0a1",
            "guid": constants.FIREFOX_APPID,
            "maxVersion": "3.*"
        }]
    }]
}


def test_between_does_not_fails_with_none():
    assert exporter.between(compare.version_int("46.2"), None, None)


def test_get_related_targetApplication_accepts_versionRange_without_targetApplication():
    assert exporter.get_related_targetApplication({}, None, None) is None


def test_plugin_record():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_plugin_items(xml_tree, [PLUGIN_DATA],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange maxVersion="4.1.10328.0" minVersion="0" severity="0" \
vulnerabilitystatus="1">
        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
          <versionRange maxVersion="3.*" minVersion="3.0a1"/>
        </targetApplication>
      </versionRange>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_plugin_items(xml_tree, [PLUGIN_DATA],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange maxVersion="4.1.10328.0" minVersion="0" severity="0" \
vulnerabilitystatus="1"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_no_guid():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 0,
        "vulnerabilityStatus": "1"
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange severity="0" vulnerabilitystatus="1"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_no_guid_and_no_vulnerability():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 1
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_no_guid_and_severity_only():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [],
        "severity": 1
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange severity="1"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_no_guid_and_severity_0():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [],
        "severity": 0
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange severity="0"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_guid_and_no_min_max_version():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"}
        ],
        "severity": 0,
        "vulnerabilityStatus": 1
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange severity="0" vulnerabilitystatus="1"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_guid_and_empty_versionRange():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"}
        ]
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_api_version_2_with_related_version():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": constants.FIREFOX_APPID,
             "minVersion": "3.6",
             "maxVersion": "3.6.*"}
        ],
        "severity": 0,
        "vulnerabilityStatus": 1
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID,
                                api_ver=2, app_ver="3.6.1")

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange severity="0" vulnerabilitystatus="1"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_no_targetApplication_info():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['name'] = "Yahoo Application State Plugin"
    data['os'] = "WINNT"
    data['xpcomabi'] = "test"
    data['versionRange'] = [{
        "targetApplication": [],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 0,
        "vulnerabilityStatus": "1"
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems>
    <pluginItem blockID="p26" name="Yahoo Application State Plugin" os="WINNT"\
 xpcomabi="test">
      <match exp="^Yahoo Application State Plugin$" name="name"/>
      <match exp="npYState.dll" name="filename"/>
      <match exp="^Yahoo Application State Plugin$" name="description"/>
      <infoURL>https://get.adobe.com/flashplayer/</infoURL>
      <versionRange maxVersion="*" minVersion="0" severity="0" \
vulnerabilitystatus="1"/>
    </pluginItem>
  </pluginItems>
</blocklist>
""".decode('utf-8')


def test_plugin_record_with_no_targetApplication_matching():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    data = PLUGIN_DATA.copy()
    data['versionRange'] = [{
        "targetApplication": [
            {"guid": "{some-other-application}",
             "minVersion": "1.2",
             "maxVersion": "1.4"}
        ],
        "minVersion": "0",
        "maxVersion": "*",
        "severity": 3
    }]

    exporter.write_plugin_items(xml_tree, [data],
                                constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <pluginItems/>
</blocklist>
""".decode('utf-8')


GFX_DATA = {
    "id": "00a6b9d2-285f-83f0-0a1f-ef0205a60067",
    "last_modified": "1348467813852",
    "blockID": "g35",
    "devices": ["0x0a6c"],
    "driverVersion": "8.17.12.5896",
    "driverVersionComparator": "LESS_THAN_OR_EQUAL",
    "feature": "DIRECT2D",
    "featureStatus": "BLOCKED_DRIVER_VERSION",
    "vendor": "0x10de",
    "versionRange": {
        "minVersion": "3.2",
        "maxVersion": "3.4"
    },
    "details": {
        "who": "All Firefox users who have these drivers installed.",
        "created": "2012-09-24T08:23:33Z",
        "name": "ATI/AMD driver 8.982.0.0",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=792480",
        "why": "Some features in these drivers are causing frequent crashes "
        "in Firefox."
    },
    "os": "WINNT 6.1"
}


def test_gfx_record():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_gfx_items(xml_tree, [GFX_DATA],
                             constants.FIREFOX_APPID)

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <gfxItems>
    <gfxBlacklistEntry blockID="g35">
      <os>WINNT 6.1</os>
      <vendor>0x10de</vendor>
      <feature>DIRECT2D</feature>
      <featureStatus>BLOCKED_DRIVER_VERSION</featureStatus>
      <driverVersion>8.17.12.5896</driverVersion>
      <driverVersionComparator>LESS_THAN_OR_EQUAL</driverVersionComparator>
      <devices>
        <device>0x0a6c</device>
      </devices>
      <versionRange minVersion="3.2" maxVersion="3.4"/>
    </gfxBlacklistEntry>
  </gfxItems>
</blocklist>
""".decode('utf-8')


CERTIFICATE_DATA = {
    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
    "last_modified": "1448372434324",
    "blockID": "c796",
    "issuerName": "MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==",
    "serialNumber": "a8V7lRiTqpdLYkrAiPw7tg==",
    "details": {
        "who": ".",
        "created": "2015-11-24T14:40:34Z",
        "name": "eDellRoot",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1227729",
        "why": "."
    }
}


def test_certificate_record():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA])

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <certItems>
    <certItem issuerName="MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==">
      <serialNumber>a8V7lRiTqpdLYkrAiPw7tg==</serialNumber>
    </certItem>
  </certItems>
</blocklist>
""".decode('utf-8')


CERTIFICATE_DATA_WITH_EMPTY_PUB_KEY = {
    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
    "last_modified": "1448372434324",
    "blockID": "c796",
    "issuerName": "MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==",
    "serialNumber": "a8V7lRiTqpdLYkrAiPw7tg==",
    "subject": "",
    "pubKeyHash": "=",
    "details": {
        "who": ".",
        "created": "2015-11-24T14:40:34Z",
        "name": "eDellRoot",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1227729",
        "why": "."
    }
}


def test_certificate_record_with_empty_pub_key():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_EMPTY_PUB_KEY])

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <certItems>
    <certItem issuerName="MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==">
      <serialNumber>a8V7lRiTqpdLYkrAiPw7tg==</serialNumber>
    </certItem>
  </certItems>
</blocklist>
""".decode('utf-8')


CERTIFICATE_DATA_WITH_SUBJECT = {
    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
    "last_modified": "1448372434324",
    "blockID": "c796",
    "subject": "MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5",
    "pubKeyHash": "VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=",
    "details": {
        "who": ".",
        "created": "2017-10-02T14:23:34Z",
        "name": "Test End-entity",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1404915",
        "why": "."
    }
}


def test_certificate_record_with_subject_and_pubKeyHash():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_SUBJECT])

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <certItems>
    <certItem pubKeyHash="VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=" \
subject="MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5"/>
  </certItems>
</blocklist>
""".decode('utf-8')


CERTIFICATE_DATA_WITH_SUBJECT_AND_EMPTY_ISSUER = {
    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
    "last_modified": "1448372434324",
    "blockID": "c796",
    "subject": "MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5",
    "pubKeyHash": "VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=",
    "issuerName": "",
    "serialNumber": "",
    "details": {
        "who": ".",
        "created": "2017-10-02T14:23:34Z",
        "name": "Test End-entity",
        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1404915",
        "why": "."
    }
}


def test_certificate_record_with_subject_and_pubKeyHash_with_empty_issuerName():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_SUBJECT_AND_EMPTY_ISSUER])

    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist">
  <certItems>
    <certItem pubKeyHash="VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=" \
subject="MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5"/>
  </certItems>
</blocklist>
""".decode('utf-8')


def test_certificate_record_post_fx58():
    xml_tree = etree.Element(
        'blocklist',
        xmlns="http://www.mozilla.org/2006/addons-blocklist",
        lastupdate='1459262434336'
    )

    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_SUBJECT_AND_EMPTY_ISSUER],
                              app_id=constants.FIREFOX_APPID, app_ver="59.0")
    result = etree.tostring(
        etree.ElementTree(xml_tree),
        pretty_print=True,
        xml_declaration=True,
        encoding='UTF-8').decode('utf-8')

    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
<blocklist lastupdate="1459262434336" \
xmlns="http://www.mozilla.org/2006/addons-blocklist"/>
""".decode('utf-8')


class TestMain(unittest.TestCase):
    def setUp(self):
        p = mock.patch('kinto_http.cli_utils.Client')
        self.MockedClient = p.start()
        self.MockedClient.return_value.get_records.return_value = []
        self.addCleanup(p.stop)

    def assert_arguments(self, kinto_client, **kwargs):
        kwargs.setdefault('kinto_server', constants.KINTO_SERVER)
        kwargs.setdefault('auth', constants.AUTH)
        kwargs.setdefault('certificates_bucket', constants.CERT_BUCKET)
        kwargs.setdefault('certificates_collection', constants.CERT_COLLECTION)
        kwargs.setdefault('gfx_bucket', constants.GFX_BUCKET)
        kwargs.setdefault('gfx_collection', constants.GFX_COLLECTION)
        kwargs.setdefault('addons_bucket', constants.ADDONS_BUCKET)
        kwargs.setdefault('addons_collection', constants.ADDONS_COLLECTION)
        kwargs.setdefault('plugins_bucket', constants.PLUGINS_BUCKET)
        kwargs.setdefault('plugins_collection', constants.PLUGINS_COLLECTION)

        kinto_client.assert_called_with(server_url=kwargs['kinto_server'],
                                        auth=kwargs['auth'],
                                        bucket=None,
                                        collection=None,
                                        retry=0,
                                        retry_after=None)

        cert_arguments = {
            'bucket': kwargs['certificates_bucket'],
            'collection': kwargs['certificates_collection'],
            'enabled': True,
            '_sort': 'last_modified',
        }

        kinto_client.return_value.get_records.assert_any_call(**cert_arguments)

        gfx_arguments = {
            'bucket': kwargs['gfx_bucket'],
            'collection': kwargs['gfx_collection'],
            'enabled': True,
            '_sort': 'last_modified',
        }

        kinto_client.return_value.get_records.assert_any_call(**gfx_arguments)

        addons_arguments = {
            'bucket': kwargs['addons_bucket'],
            'collection': kwargs['addons_collection'],
            'enabled': True,
            '_sort': 'last_modified',
        }

        kinto_client.return_value.get_records.assert_any_call(
            **addons_arguments)

        plugins_arguments = {
            'bucket': kwargs['plugins_bucket'],
            'collection': kwargs['plugins_collection'],
            'enabled': True,
            '_sort': 'last_modified',
        }

        kinto_client.return_value.get_records.assert_any_call(
            **plugins_arguments)

    def test_main_default(self):
        # let's check that main() parsing uses our defaults
        exporter.main([])
        self.assert_arguments(self.MockedClient)

    def test_main_custom_server(self):
        exporter.main(['-s', 'http://yeah'])
        self.assert_arguments(self.MockedClient, kinto_server='http://yeah')

    def test_can_define_the_certificates_bucket_and_collection(self):
        exporter.main(['--certificates-bucket', 'bucket',
                       '--certificates-collection', 'collection'])
        self.assert_arguments(self.MockedClient,
                              certificates_bucket='bucket',
                              certificates_collection='collection')

    def test_can_define_the_gfx_bucket_and_collection(self):
        exporter.main(['--gfx-bucket', 'bucket',
                       '--gfx-collection', 'collection'])
        self.assert_arguments(self.MockedClient,
                              gfx_bucket='bucket',
                              gfx_collection='collection')

    def test_can_define_the_addons_bucket_and_collection(self):
        exporter.main(['--addons-bucket', 'bucket',
                       '--addons-collection', 'collection'])
        self.assert_arguments(self.MockedClient,
                              addons_bucket='bucket',
                              addons_collection='collection')

    def test_can_define_the_plugins_bucket_and_collection(self):
        exporter.main(['--plugins-bucket', 'bucket',
                       '--plugins-collection', 'collection'])
        self.assert_arguments(self.MockedClient,
                              plugins_bucket='bucket',
                              plugins_collection='collection')

    def test_can_define_the_auth_credentials(self):
        exporter.main(['--auth', 'user:pass'])
        self.assert_arguments(self.MockedClient, auth=('user', 'pass'))

    def test_can_define_the_output_file(self):
        out_file = StringIO()
        with mock.patch.object(out_file, 'close') as mocked_close:
            with mock.patch('amo2kinto.exporter.open', return_value=out_file):
                exporter.main(['--out', 'file'])
                self.assert_arguments(self.MockedClient)

                self.assertTrue(mocked_close.called)
                self.assertNotEqual(len(out_file.getvalue()), 0)

    def test_get_records_read_fails(self):
        with mock.patch('kinto_http.cli_utils.Client') as MockedClient:
            MockedClient.return_value.get_records.side_effect = Exception
            with mock.patch('amo2kinto.exporter.logger') as mocked_logger:
                exporter.main([])
                self.assertEqual(mocked_logger.warn.call_count, 4)

    def test_last_updated_uses_greater_value(self):
        self.MockedClient.return_value.get_records.side_effect = (
            [ADDONS_DATA], [PLUGIN_DATA], [GFX_DATA], [CERTIFICATE_DATA]
        )
        out_file = StringIO()
        with mock.patch.object(out_file, 'close'):
            with mock.patch('amo2kinto.exporter.open', return_value=out_file):
                exporter.main(['--out', 'file'])
        self.assertIn('lastupdate="1448372434324"', out_file.getvalue())

    def test_no_certs_post_fx58(self):
        self.MockedClient.return_value.get_records.side_effect = (
            [ADDONS_DATA], [PLUGIN_DATA], [GFX_DATA], [CERTIFICATE_DATA]
        )
        out_file = StringIO()
        with mock.patch.object(out_file, 'close'):
            with mock.patch('amo2kinto.exporter.open', return_value=out_file):
                exporter.main(['--out', 'file', '--app', constants.FIREFOX_APPID,
                               '--app-version', '58.0'])
        self.assertNotIn('<certItems>', out_file.getvalue())



from amo2kinto.amo import make_id_from_string


def test_make_id_from_string():
    """It should handle strings, list, int and dict."""
    assert make_id_from_string("foo") == 'acbd18db-4cc2-f85c-edef-654fccc4a4d8'



import os
import datetime

from dateutil.parser import parse as dateutil_parser
from jinja2 import Environment, PackageLoader
from kinto_http import cli_utils

from . import constants
from .logger import logger


JSON_DATE_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
COLLECTION_FORMAT = '/buckets/{bucket_id}/collections/{collection_id}'


def iso2human(value):
    date = dateutil_parser(value)
    return date.strftime('%b %d, %Y')


def get_template(name):
    loader = PackageLoader('amo2kinto', 'templates')
    env = Environment(loader=loader)
    env.filters['datetime'] = iso2human
    return env.get_template(name)


def generate_index(records, template, target_dir):

    def sort_records(record):
        return record['details']['created'], record['last_modified']

    records = sorted(records, key=sort_records, reverse=True)
    res = template.render(records=records)
    filename = os.path.join(target_dir, 'index.html')

    logger.info('Writing %s' % filename)
    with open(filename, 'w') as f:
        f.write(res.encode('utf8'))


def generate_record(record, template, target_dir, get_filename):
    res = template.render(record=record)
    filename = os.path.join(target_dir, get_filename(record))

    logger.info('Writing %s' % filename)

    # This needs to be able to write to S3
    with open(filename, 'w') as f:
        f.write(res.encode('utf-8'))


def get_record_filename(record):
    return '%s.html' % record.get('blockID', record['id'])


def generate(client, collections, target_dir, collection_template, record_template):
    logger.info('Reading data from Kinto %s...' % client.session.server_url)
    records = []

    for collection_uri in collections:
        parts = collection_uri.strip('/').split('/')
        bucket_id = parts[1]
        collection_id = parts[3]

        logger.warn('Reading records from %s...' % collection_uri)
        response = client.get_records(bucket=bucket_id, collection=collection_id)
        for record in response:
            record.setdefault('details', {})
            if 'created' not in record['details']:
                date = datetime.date.fromtimestamp(record['last_modified'] / 1000)
                record['details']['created'] = date.strftime(JSON_DATE_FORMAT)
            records.append(record)

    if not os.path.exists(target_dir):
        os.makedirs(target_dir)

    col_tpl = get_template(collection_template)
    rec_tpl = get_template(record_template)

    logger.warn('Generating index...')
    generate_index(records, col_tpl, target_dir)

    logger.warn('Generating records files...')
    for record in records:
        generate_record(record, rec_tpl, target_dir, get_record_filename)


def main(args=None):
    parser = cli_utils.add_parser_options(
        description='Generate blocked item description files.',
        default_collection=None,
        default_bucket=constants.DESTINATION_BUCKET,
        default_server=constants.KINTO_SERVER,
        default_auth=constants.AUTH,
        include_collection=False)

    parser.add_argument('--addons-collection',
                        help='Collection name for addon',
                        type=str, default=constants.ADDONS_COLLECTION)

    parser.add_argument('--plugins-collection',
                        help='Collection name for plugin',
                        type=str, default=constants.PLUGINS_COLLECTION)

    parser.add_argument('-d', '--target-dir',
                        help='Destination directory to write files in.',
                        type=str, default=constants.TARGET_DIR)

    args = parser.parse_args(args=args)
    cli_utils.setup_logger(logger, args)

    kinto_client = cli_utils.create_client_from_args(args)

    collections = [
        COLLECTION_FORMAT.format(bucket_id=args.bucket,
                                 collection_id=args.addons_collection),
        COLLECTION_FORMAT.format(bucket_id=args.bucket,
                                 collection_id=args.plugins_collection),
    ]
    generate(kinto_client, collections, args.target_dir, 'collection.tpl', 'record.tpl')



import codecs
import os
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(HERE, 'README.rst'), encoding='utf-8') as f:
    README = f.read()

with codecs.open(os.path.join(HERE, 'CHANGELOG.rst'), encoding='utf-8') as f:
    CHANGELOG = f.read()


REQUIREMENTS = [
    'jinja2',
    'kinto-http>=8',
    'lxml',
    'python-dateutil',
]


ENTRY_POINTS = {
    'console_scripts': [
        'kinto2xml = amo2kinto.exporter:main',
        'blockpages-generator = amo2kinto.generator:main',
    ]
}


setup(name='amo2kinto',
      version='3.2.0',
      description='Generate a blocklists.xml file from the Kinto collections.',
      long_description=README + "\n\n" + CHANGELOG,
      license='Apache License (2.0)',
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "License :: OSI Approved :: Apache Software License"
      ],
      keywords="web services",
      author='Mozilla Services',
      author_email='services-dev@mozilla.com',
      url='https://github.com/mozilla-services/amo2kinto',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=REQUIREMENTS,
      entry_points=ENTRY_POINTS)
